`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    11:46:52 11/30/2017 
// Design Name: 
// Module Name:    ti_cpld_a 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module ti_cpld_a(
		input [15:0] bus_a,
		input [5:0]  ti_in,
		input nRD,
		input nWR,
		input nMREQ,
		input nIORQ,
		input nM1,
		
		inout  [7:0] bus_d,
		
		output nRAMCE,
		output nROMCE,
		output ROMCS,
		output reg [5:0] ti_out
    );

initial
	ti_out = 6'b000000;

// M397 składa się z dwóch niezależnych bloków funkcjonalnych. 
// Jeden obsługuje port I/O, drugi zarządza pamięcią.

// IO - port 0xef = 0b11101111

	// true jeśli komputer odczytuje port 0xef
	wire ioRead = ((nRD == 0) && (nIORQ == 0) && ( bus_a[7:0] == 8'hef)); 
	// true jeśli komputer pisze do portu 0xef
	wire ioWrite = ((nWR == 0) && (nIORQ == 0) && ( bus_a[7:0] == 8'hef)); 

	// JAK działa interface?
	// Jeśli wartość sygnału ioRead jest 'prawdą',
	// to wystaw na szynę danych komputera wartość odczytaną z portu interfejsu.
	// W przeciwnym przypadku, szyna danych przechodzi w stan wysoki.
	assign bus_d[7:0] = (ioRead)?{1'b0,ti_in[5],1'b0, ti_in[4:0] } : 8'bzzzzzzzz;

	// Jeśli zmieni się wartość sygnału ioWrite i jest ona 'prawdą',
	// to wyślij do stacji wartość z szyny danych.
	// W przeciwnym przypadku, pozostaw ją bez zmian.
	// można tak jak poniżej :
	// assign ti_out[5:0] = (ioWrite)?{ bus_d[6],bus_d[4:0] }:ti_out[5:0];
	// lub tak
	always@(*)
		if (ioWrite) ti_out[5:0] = { bus_d[6],bus_d[4:0] };
	// oba rozwiązania generują ostrzeżenie o utworzonym zatrzasku.
	
// PAMIĘĆ - adresy 0x00, 0x08, 0x604
	
	// mPaged = 1 - podmieniliśmy pamięci na te z interfejsu.
	reg mPaged = 0;
	// Trzeba odłączyć oryginalny ROM
	assign ROMCS  = mPaged;
	// jeśli pierwsze 8k pamięci - aktywujemy ROM
	assign nROMCE = !((bus_a[15:13] == 3'b000) && mPaged);
	// jeśli drugie 8k pamięci - aktywujemy RAM
	assign nRAMCE = !((bus_a[15:13] == 3'b001) && mPaged);
	
	// true jeśli komputer odczytuje kod rozkazu
	wire mRead = ((nRD == 0) && (nMREQ == 0) && (nM1 == 0)); 
	// Jeśli odczytywany jest kod rozkazu spod adresu 0 lub 8 - przełączamy pamięci.
	// jeśli odczytywany jest kod rozkazu spod adresu 0x604 - załączamy oryginalny ROM.
	always@(*)
		if (mRead) begin
			if    ( (bus_a[15:0] == 16'h0000)||(bus_a[15:0] == 16'h0008)) mPaged = 1;
			else if (bus_a[15:0] == 16'h0604) mPaged = 0;
		end
   // Ten kod także wygeneruje ostrzeżenie o zatrzasku. Uroki układów asynchronicznych.
	
	// I to wszystko. Jak widać definicja interfejsu składa się głównie z komentarzy :)
	
endmodule

module ti_cpld_s(
		input [15:0] bus_a,
		input [5:0]  ti_in,
		input nRD,
		input nWR,
		input nMREQ,
		input nIORQ,
		input nM1,
		
		inout  [7:0] bus_d,
		
		output nRAMCE,
		output nROMCE,
		output reg ROMCS,
		output reg [5:0] ti_out
    );

	initial begin
		ti_out[5:0] = 6'd0;
		ROMCS = 0;
	end

	wire portRD = (nIORQ==0)&&(nRD==0)&&(bus_a[7:0] == 8'hef);
	wire portWR = (nIORQ==0)&&(nWR==0)&&(bus_a[7:0] == 8'hef);

	always @(posedge portWR) ti_out <= { bus_d[6],bus_d[4:0] };
	assign bus_d = (portRD)?{1'b0,ti_in[5],1'b0, ti_in[4:0] }:8'hzz;

	wire memRD  = (nMREQ==0)&&(nRD==0);
	wire memWR  = (nMREQ==0)&&(nWR==0);
	wire memEXE = memRD&&(nM1==0);


	always@(posedge memEXE)
		if ((bus_a == 16'h0)||(bus_a == 16'h8)) ROMCS<=1;
		else if (bus_a == 16'h0604) ROMCS <=0;

	wire addrROM = (bus_a[15:13] == 3'b000);
	wire addrRAM = (bus_a[15:13] == 3'b001);

	assign nROMCE = ~(ROMCS&addrROM&(memRD|memWR));
	assign nRAMCE = ~(ROMCS&addrRAM&(memRD|memWR));

endmodule


module ti_cpld_c(
		input [15:0] bus_a,
		input [5:0]  ti_in,
		input nRD,
		input nWR,
		input nMREQ,
		input nIORQ,
		input nM1,
		
		inout  [7:0] bus_d,
		
		output reg nRAMCE,
		output reg nROMCE,
		output reg ROMCS,
		output reg [5:0] ti_out
    );

initial begin
	ti_out = 6'b000000;
	ROMCS = 0;
end

	reg OE = 0;
	assign bus_d = (OE) ? {1'b0,ti_in[5],1'b0, ti_in[4:0] } : 8'bzzzzzz;
	
	wire a_page_in  = ((bus_a == 16'h0) || (bus_a == 16'h8));
	wire a_page_out =  (bus_a == 16'h0604);
	wire a_port_io  =  (bus_a[7:0] == 8'hef);
	
	wire a_ROM = (bus_a[15:13] == 3'b000);
	wire a_RAM = (bus_a[15:13] == 3'b001);
	
	always@(*) begin
		case ({a_page_in,a_page_out,nRD,nMREQ,nM1})
			5'b10000 : ROMCS <= 1;
			5'b01000 : ROMCS <= 0;
		endcase
		
		case ({ROMCS,a_ROM,a_RAM,nRD,nWR,nMREQ})
			6'b110010,
			6'b110100 : nROMCE <= 0;
			6'b101010,
			6'b101100 : nRAMCE <= 0;
			default	 : begin nROMCE <= 1; nRAMCE <= 1; end
		endcase
	
		case ({a_port_io,nRD,nWR,nIORQ})
			4'b1100 : ti_out <=  { bus_d[6],bus_d[4:0] };
			4'b1010 : OE <= 1;
			default : OE <= 0;
		endcase
	end
	
endmodule
	